@@ -40,10 +40,11 @@ c
 *     Modified by J.K. Lindsey for R, March, 1998, November, 1998       *
 *************************************************************************
       implicit none
-      integer n(1),m,iq,i,j,k,l,model,nobs(1),mobs,ii,nm,pos(m),nn
+      integer n(1),m,iq,i,j,k,l,model,nobs(1),mobs,ii,nm,pos(m),nn,
+     +     pivot(m)
       logical cf
       double precision like,s(1),pi,sflog,av,pshape(m),
-     +     gmod(m,m),rhs(m),pivot(m),qraux(m),work(2*m),cmu(iq,m,l),
+     +     gmod(m,m),rhs(m),qraux(m),work(2*m),cmu(iq,m,l),
      +     tvmu(mobs,m,l),ll,x(1),gamma(m,m),delta(m),a(m),b(m,m),
      +     c(m),pgamma(m,m),filter(m,nn),pfam
       double precision bernpr,poispr,multpr,binpr,exppr,bbinpr,nbinpr,
@@ -296,8 +297,8 @@ c transform back to original values
 * Computes stationary distribution of Markov chain *
 ****************************************************
       implicit none
-      integer m,i,j,rank,info
-      double precision pivot(1),qraux(1),work(1),
+      integer m,i,j,rank,info,pivot(1)
+      double precision qraux(1),work(1),
      +     gamma(m,m),delta(1),gmod(m,m),rhs(1)
 
       do 20 i = 2, m
@@ -309,7 +310,7 @@ c transform back to original values
  20   continue
 *
       do 30 j = 1, m
-         pivot(i)=j
+         pivot(j)=j
          gmod(1,j) = 1.
  30   continue
       rhs(1) = 1.
@@ -640,8 +641,8 @@ c transform back to original values
       double precision svec,pvec,pshape,pfam
 
       burrpr=svec/pvec
-      burrpr=dlog(pshape*pfam/pvec)+(pshape-1.0)*dlog(burrpr)-
-     +	    (pfam+1.0)*dlog(1.0+burrpr**pshape)
+      burrpr=dlog(pshape/pvec)+(pshape-1.0)*dlog(burrpr)-
+     +	    (pfam+1.0)*dlog(1.0+burrpr**pshape/pfam)
       return
       end
 
@@ -665,7 +666,7 @@ c transform back to original values
       implicit none
       double precision svec,pvec,pshape,pfam,tmp
 
-      if(pfam>0)then
+      if(pfam.gt.0)then
          tmp=-pvec**(-pshape)
       else
          tmp=dlog(1.0-dexp(-pvec**(-pshape)))
